@@ -21,16 +21,11 @@
 Frescobaldi main menu.
 """
 
-from __future__ import unicode_literals
 
-import sys
-if sys.version_info >= (3, 0):
-    import builtins # py3
-else:
-    import __builtin__ as builtins # py2
+import builtins
 
-from PyQt4.QtCore import QSettings
-from PyQt4.QtGui import QMenu
+from PyQt5.QtCore import QSettings
+from PyQt5.QtWidgets import QMenu
 
 import app
 import icons
@@ -52,7 +47,7 @@ import matcher
 import file_import
 import file_export
 import browseriface
-import vcs
+import fonts
 
 
 # postpone translation
@@ -62,7 +57,7 @@ _ = lambda *args: lambda: builtins._(*args)
 def createMenus(mainwindow):
     """Adds all the menus to the mainwindow's menubar."""
     m = mainwindow.menuBar()
-    
+
     m.addMenu(menu_file(mainwindow))
     m.addMenu(menu_edit(mainwindow))
     m.addMenu(menu_view(mainwindow))
@@ -73,7 +68,7 @@ def createMenus(mainwindow):
     m.addMenu(menu_document(mainwindow))
     m.addMenu(menu_window(mainwindow))
     m.addMenu(menu_session(mainwindow))
-    if vcs.app_is_git_controlled():
+    if app.is_git_controlled():
         from vcs.menu import GitMenu
         m.addMenu(GitMenu(mainwindow))
     m.addMenu(menu_help(mainwindow))
@@ -84,9 +79,10 @@ class Menu(QMenu):
     def __init__(self, title_func, parent=None):
         """title_func should return the title for the menu when called."""
         super(Menu, self).__init__(parent)
+        self.setToolTipsVisible(True)
         self.title_func = title_func
         app.translateUI(self)
-    
+
     def translateUI(self):
         self.setTitle(self.title_func())
 
@@ -94,64 +90,76 @@ class Menu(QMenu):
 def menu_file(mainwindow):
     m = Menu(_("menu title", "&File"), mainwindow)
     ac = mainwindow.actionCollection
-    
+
     m.addAction(ac.file_new)
-    m.addAction(scorewiz.ScoreWizard.instance(mainwindow).actionCollection.scorewiz)
     m.addMenu(snippet.menu.TemplateMenu(mainwindow))
     m.addSeparator()
     m.addAction(ac.file_open)
     m.addAction(ac.file_open_recent)
-    m.addAction(ac.file_insert_file)
-    m.addSeparator()
+    m.addAction(ac.file_close)
+    m.addMenu(menu_file_close(mainwindow))
     m.addAction(ac.file_save)
-    m.addAction(ac.file_save_as)
-    m.addAction(ac.file_save_copy_as)
-    m.addAction(panelmanager.manager(mainwindow).snippettool.actionCollection.file_save_as_template)
-    m.addAction(ac.file_save_all)
+    m.addMenu(menu_file_save(mainwindow))
+    m.addSeparator()
+    m.addMenu(menu_file_import(mainwindow))
+    m.addMenu(menu_file_print(mainwindow))
     m.addSeparator()
     m.addAction(ac.file_reload)
     m.addAction(ac.file_reload_all)
     m.addAction(ac.file_external_changes)
     m.addSeparator()
-    m.addMenu(menu_file_import(mainwindow))
-    m.addMenu(menu_file_export(mainwindow))
-    m.addSeparator()
-    m.addAction(panelmanager.manager(mainwindow).musicview.actionCollection.music_print)
-    m.addAction(ac.file_print_source)
-    m.addSeparator()
-    m.addAction(ac.file_close)
-    m.addAction(ac.file_close_other)
-    m.addAction(ac.file_close_all)
-    m.addSeparator()
     m.addAction(ac.file_quit)
-    if vcs.app_is_git_controlled():
+    if app.is_git_controlled():
         m.addAction(ac.file_restart)
     return m
 
 
-def menu_file_import(mainwindow):
-    m = Menu(_("submenu title", "&Import"), mainwindow)
-    ac = file_import.FileImport.instance(mainwindow).actionCollection
-    
-    m.addAction(ac.import_all)
-    m.addSeparator()
-    m.addAction(ac.import_musicxml)
-    m.addAction(ac.import_midi)
-    m.addAction(ac.import_abc)
+def menu_file_save(mainwindow):
+    m = Menu(_("submenu title", "Save"), mainwindow)
+    ac = mainwindow.actionCollection
+    m.addAction(ac.file_save_as)
+    m.addAction(ac.file_save_copy_as)
+    m.addAction(ac.file_rename)
+    m.addAction(panelmanager.manager(mainwindow).snippettool.actionCollection.file_save_as_template)
+    m.addAction(ac.file_save_all)
     return m
 
 
-def menu_file_export(mainwindow):
-    m = Menu(_("submenu title", "&Export"), mainwindow)
+def menu_file_close(mainwindow):
+    m = Menu(_("submenu title", "Close"), mainwindow)
     ac = mainwindow.actionCollection
+    m.addAction(ac.file_close_other)
+    m.addAction(ac.file_close_all)
+    m.addAction(ac.file_close_all_and_session)
+    return m
+
+
+def menu_file_import(mainwindow):
+    m = Menu(_("submenu title", "&Import/Export"), mainwindow)
+    ac = mainwindow.actionCollection
+    acfi = file_import.FileImport.instance(mainwindow).actionCollection
     acfe = file_export.FileExport.instance(mainwindow).actionCollection
-    
-    if vcs.app_is_git_controlled() or QSettings().value("experimental-features", False, bool):
+
+    m.addAction(acfi.import_any)
+    m.addSeparator()
+    m.addAction(acfi.import_musicxml)
+    m.addAction(acfi.import_midi)
+    m.addAction(acfi.import_abc)
+    m.addSeparator()
+    if app.is_git_controlled() or QSettings().value("experimental-features", False, bool):
         m.addAction(acfe.export_audio)
         m.addAction(acfe.export_musicxml)
     m.addAction(ac.export_colored_html)
     return m
-    
+
+
+def menu_file_print(mainwindow):
+    m = Menu(_("submenu title", "&Print"), mainwindow)
+    ac = mainwindow.actionCollection
+    m.addAction(panelmanager.manager(mainwindow).musicview.actionCollection.music_print)
+    m.addAction(ac.file_print_source)
+    return m
+
 
 def menu_edit(mainwindow):
     m = Menu(_("menu title", "&Edit"), mainwindow)
@@ -160,13 +168,11 @@ def menu_edit(mainwindow):
     m.addAction(ac.edit_undo)
     m.addAction(ac.edit_redo)
     m.addSeparator()
-    m.addAction(documentactions.get(mainwindow).actionCollection.edit_cut_assign)
-    m.addAction(documentactions.get(mainwindow).actionCollection.edit_move_to_include_file)
     m.addAction(ac.edit_cut)
     m.addAction(ac.edit_copy)
-    m.addAction(panelmanager.manager(mainwindow).snippettool.actionCollection.copy_to_snippet)
-    m.addAction(ac.edit_copy_colored_html)
     m.addAction(ac.edit_paste)
+    m.addAction(ac.file_insert_file)
+    m.addMenu(menu_edit_cut(mainwindow))
     m.addSeparator()
     m.addAction(ac.edit_select_all)
     m.addAction(ac.edit_select_current_toplevel)
@@ -178,13 +184,23 @@ def menu_edit(mainwindow):
     m.addAction(ac.edit_replace)
     m.addSeparator()
     m.addAction(ac.edit_preferences)
-    return m    
+    return m
+
+
+def menu_edit_cut(mainwindow):
+    m = Menu(_("menu title", "Cut/Copy (advanced)"), mainwindow)
+    ac = mainwindow.actionCollection
+    m.addAction(documentactions.get(mainwindow).actionCollection.edit_cut_assign)
+    m.addAction(documentactions.get(mainwindow).actionCollection.edit_move_to_include_file)
+    m.addAction(panelmanager.manager(mainwindow).snippettool.actionCollection.copy_to_snippet)
+    m.addAction(ac.edit_copy_colored_html)
+    return m
 
 
 def menu_view(mainwindow):
     m = Menu(_("menu title", "&View"), mainwindow)
     ac = mainwindow.actionCollection
-    
+
     m.addAction(ac.view_next_document)
     m.addAction(ac.view_previous_document)
     m.addSeparator()
@@ -194,6 +210,7 @@ def menu_view(mainwindow):
     m.addMenu(menu_view_folding(mainwindow))
     m.addSeparator()
     m.addAction(documentactions.get(mainwindow).actionCollection.view_goto_file_or_definition)
+    m.addAction(ac.view_goto_line)
     ac = browseriface.get(mainwindow).actionCollection
     m.addAction(ac.go_back)
     m.addAction(ac.go_forward)
@@ -218,7 +235,7 @@ def menu_view(mainwindow):
 def menu_view_folding(mainwindow):
     m = Menu(_("submenu title", "&Folding"), mainwindow)
     ac = sidebar.SideBarManager.instance(mainwindow).actionCollection
-    
+
     m.addAction(ac.folding_enable)
     m.addSeparator()
     m.addAction(ac.folding_fold_current)
@@ -233,7 +250,7 @@ def menu_view_folding(mainwindow):
 def menu_music(mainwindow):
     m = Menu(_("menu title", "&Music"), mainwindow)
     ac = panelmanager.manager(mainwindow).musicview.actionCollection
-    
+
     m.addAction(ac.music_reload)
     m.addSeparator()
     m.addAction(ac.music_zoom_in)
@@ -247,6 +264,14 @@ def menu_music(mainwindow):
     m.addAction(ac.music_single_pages)
     m.addAction(ac.music_two_pages_first_right)
     m.addAction(ac.music_two_pages_first_left)
+    m.addAction(ac.music_raster)
+    m.addSeparator()
+    m.addAction(ac.music_horizontal)
+    m.addAction(ac.music_vertical)
+    m.addAction(ac.music_continuous)
+    m.addSeparator()
+    m.addAction(ac.music_rotate_right)
+    m.addAction(ac.music_rotate_left)
     m.addSeparator()
     m.addAction(ac.music_copy_image)
     m.addAction(ac.music_copy_text)
@@ -255,6 +280,7 @@ def menu_music(mainwindow):
     m.addAction(ac.music_sync_cursor)
     m.addSeparator()
     m.addAction(ac.music_maximize)
+    m.addAction(ac.music_save_settings)
     return m
 
 
@@ -265,7 +291,7 @@ def menu_snippets(mainwindow):
 def menu_lilypond(mainwindow):
     m = Menu(_("menu title", "&LilyPond"), mainwindow)
     ac = engrave.engraver(mainwindow).actionCollection
-    
+
     m.addAction(ac.engrave_sticky)
     m.addAction(ac.engrave_autocompile)
     m.addSeparator()
@@ -276,9 +302,6 @@ def menu_lilypond(mainwindow):
     m.addAction(ac.engrave_abort)
     m.addSeparator()
     m.addMenu(menu_lilypond_generated_files(mainwindow))
-    m.addSeparator()
-    m.addAction(ac.engrave_open_lilypond_datadir)
-    m.addAction(ac.engrave_show_available_fonts)
     return m
 
 
@@ -288,32 +311,53 @@ def menu_lilypond_generated_files(mainwindow):
 
 def menu_tools(mainwindow):
     m = Menu(_('menu title', '&Tools'), mainwindow)
-    
+
+    ac = autocomplete.CompleterManager.instance(mainwindow).actionCollection
+    m.addAction(ac.autocomplete)
+    m.addAction(ac.popup_completions)
+    m.addSeparator()
+    m.addMenu(menu_tools_format(mainwindow))
+    m.addMenu(menu_tools_transform(mainwindow))
+    fac = fonts.fonts(mainwindow).actionCollection
+    m.addAction(fac.fonts_document_fonts)
+    dac = documentactions.get(mainwindow).actionCollection
+    m.addAction(dac.tools_convert_ly)
+    m.addSeparator()
+    m.addMenu(menu_tools_directories(mainwindow))
+    m.addSeparator()
+    panelmanager.manager(mainwindow).addActionsToMenu(m)
+    extensions = app.extensions()
+    # store a reference to the Tools menu
+    extensions.set_tools_menu(m)
+    ext_menu = extensions.menu('tools')
+
+    if not ext_menu.isEmpty():
+        m.addSeparator()
+        m.addMenu(ext_menu)
+    return m
+
+
+def menu_tools_format(mainwindow):
+    m = Menu(_('submenu title', "Code &Formatting"), mainwindow)
+    m.setIcon(icons.get('document-edit'))
     ac = documentactions.get(mainwindow).actionCollection
     m.addAction(ac.tools_indent_auto)
     m.addAction(ac.tools_indent_indent)
     m.addAction(ac.tools_reformat)
     m.addAction(ac.tools_remove_trailing_whitespace)
-    m.addSeparator()
-    ac = autocomplete.CompleterManager.instance(mainwindow).actionCollection
-    m.addAction(ac.autocomplete)
-    m.addAction(ac.popup_completions)
-    m.addSeparator()
+    return m
+
+def menu_tools_transform(mainwindow):
+    m = Menu(_('submenu title', "Musical &Transformations"), mainwindow)
+    m.setIcon(icons.get('Audio-x-generic'))
     m.addMenu(menu_tools_pitch(mainwindow))
     m.addMenu(menu_tools_rest(mainwindow))
     m.addMenu(menu_tools_rhythm(mainwindow))
     m.addMenu(menu_tools_lyrics(mainwindow))
+    m.addMenu(menu_tools_directions(mainwindow))
     m.addMenu(menu_tools_quick_remove(mainwindow))
-    m.addSeparator()
-    ac = documentactions.get(mainwindow).actionCollection
-    m.addAction(ac.tools_convert_ly)
-    m.addAction(mainwindow.actionCollection.file_open_current_directory)
-    m.addAction(mainwindow.actionCollection.file_open_command_prompt)
-    m.addSeparator()
-    panelmanager.manager(mainwindow).addActionsToMenu(m)
     return m
 
-
 def menu_tools_lyrics(mainwindow):
     m = Menu(_('submenu title', "&Lyrics"), mainwindow)
     m.setIcon(icons.get('audio-input-microphone'))
@@ -330,7 +374,7 @@ def menu_tools_pitch(mainwindow):
     m = Menu(_('submenu title', "&Pitch"), mainwindow)
     m.setIcon(icons.get('tools-pitch'))
     ac = pitch.Pitch.instance(mainwindow).actionCollection
-    
+
     m.addAction(ac.pitch_language)
     m.addSeparator()
     m.addAction(ac.pitch_relative_assume_first_pitch_absolute)
@@ -349,7 +393,7 @@ def menu_tools_rest(mainwindow):
     m = Menu(_('submenu title', "Rest"), mainwindow)
     m.setIcon(icons.get('tools-rest'))
     ac = rest.Rest.instance(mainwindow).actionCollection
-    
+
     m.addAction(ac.rest_fmrest2spacer)
     m.addAction(ac.rest_spacer2fmrest)
     m.addSeparator()
@@ -361,7 +405,7 @@ def menu_tools_rhythm(mainwindow):
     m = Menu(_('submenu title', "&Rhythm"), mainwindow)
     m.setIcon(icons.get('tools-rhythm'))
     ac = rhythm.Rhythm.instance(mainwindow).actionCollection
-    
+
     m.addAction(ac.rhythm_double)
     m.addAction(ac.rhythm_halve)
     m.addSeparator()
@@ -382,11 +426,21 @@ def menu_tools_rhythm(mainwindow):
     return m
 
 
+def menu_tools_directions(mainwindow):
+    m = Menu(_('submenu title', "&Directions"), mainwindow)
+    m.setIcon(icons.get('music-directions'))
+    ac = documentactions.DocumentActions.instance(mainwindow).actionCollection
+    m.addAction(ac.tools_directions_force_up)
+    m.addAction(ac.tools_directions_force_neutral)
+    m.addAction(ac.tools_directions_force_down)
+    return m
+
+
 def menu_tools_quick_remove(mainwindow):
     m = Menu(_('submenu title', "&Quick Remove"), mainwindow)
     m.setIcon(icons.get('edit-clear'))
     ac = documentactions.DocumentActions.instance(mainwindow).actionCollection
-    
+
     m.addAction(ac.tools_quick_remove_comments)
     m.addAction(ac.tools_quick_remove_articulations)
     m.addAction(ac.tools_quick_remove_ornaments)
@@ -399,6 +453,15 @@ def menu_tools_quick_remove(mainwindow):
     m.addAction(ac.tools_quick_remove_markup)
     return m
 
+def menu_tools_directories(mainwindow):
+    m = Menu(_('submenu title', "&Directories"), mainwindow)
+    m.setIcon(icons.get('folder-open'))
+    m.addAction(mainwindow.actionCollection.file_open_current_directory)
+    m.addAction(mainwindow.actionCollection.file_open_command_prompt)
+    m.addSeparator()
+    ac = engrave.engraver(mainwindow).actionCollection
+    m.addAction(ac.engrave_open_lilypond_datadir)
+    return m
 
 def menu_document(mainwindow):
     return documentmenu.DocumentMenu(mainwindow)
@@ -437,5 +500,3 @@ def menu_help(mainwindow):
     m.addSeparator()
     m.addAction(ac.help_about)
     return m
-
-
